#include <analysis/Polynomial1DRootComputer.hpp>

double
Polynomial1DRootComputer::_computeRootBetween(const PositionSignChanges& left_bound,
                                              const PositionSignChanges& right_bound) const
{
  double left  = left_bound.position;
  double right = right_bound.position;

  const double interval_size = m_right - m_left;

  const Polynomial1D& p0 = m_sturm_sequence.p(0);
  double left_value      = p0(left);

  int cpt = 0;
  while (right - left > 1E-15 * interval_size) {
    double middle       = 0.5 * (right + left);
    double middle_value = p0(middle);
    if (middle_value * left_value < 0) {
      right = middle;
    } else {
      left_value = middle_value;
      left       = middle;
    }
    cpt++;
  }

  return 0.5 * (left + right);
}

std::optional<double>
Polynomial1DRootComputer::getFirstRoot() const
{
  PositionSignChanges l = {m_left, m_sturm_sequence.numberOfSignChanges(m_left)};
  PositionSignChanges r = {m_right, m_sturm_sequence.numberOfSignChanges(m_right)};

  if (l.nb_sign_changes - r.nb_sign_changes > 0) {
    if (l.nb_sign_changes - r.nb_sign_changes > 1) {
      PositionSignChanges a = l;
      PositionSignChanges b = r;
      do {
        Assert(a.nb_sign_changes - b.nb_sign_changes > 0);

        double position_k      = 0.5 * (a.position + b.position);
        PositionSignChanges rk = {position_k, m_sturm_sequence.numberOfSignChanges(position_k)};

        if (l.nb_sign_changes - rk.nb_sign_changes > 0) {
          b = rk;
        } else if (l.nb_sign_changes - rk.nb_sign_changes == 0) {
          a = rk;
        }
        if (b.position - a.position < 1E-14 * (m_right - m_left)) {
          return a.position;
        }
      } while (a.nb_sign_changes - b.nb_sign_changes != 1);
      return _computeRootBetween(a, b);
    } else {
      return _computeRootBetween(l, r);
    }
  } else {
    return {};
  }
}

Polynomial1DRootComputer::Polynomial1DRootComputer(const Polynomial1D& p, const double a, const double b)
  : m_sturm_sequence{p}, m_left{a}, m_right{b}
{}
