#include <analysis/SturmSequence.hpp>

SturmSequence::SturmSequence(const Polynomial1D& p0) : SturmSequence{Polynomial1D{p0}} {}

SturmSequence::SturmSequence(Polynomial1D&& p0)
{
  m_sequence.reserve(p0.degree());
  m_sequence.emplace_back(p0);
  Polynomial1D p1 = derive(m_sequence[0]);
  if (p1.degree() > 0 or (std::abs(p1.coefficient(0)) > 1E-14)) {
    m_sequence.emplace_back(std::move(p1));

    do {
      Polynomial1D pk = -(m_sequence[m_sequence.size() - 2] % m_sequence[m_sequence.size() - 1]);
      if (pk.degree() > 0 or (std::abs(pk.coefficient(0)) > 1E-14)) {
        m_sequence.emplace_back(std::move(pk));
      } else {
        break;
      }
    } while (true);
  }
}

size_t
SturmSequence::numberOfSignChanges(const double x) const
{
  std::vector<double> values;
  values.reserve(m_sequence.size());
  for (const auto& pk : m_sequence) {
    values.emplace_back(pk(x));
  }

  size_t number_of_sign_changes = 0;
  for (size_t i = 1; i < m_sequence.size(); ++i) {
    number_of_sign_changes += (values[i - 1] * values[i] < 0);
  }

  return number_of_sign_changes;
}
