#ifndef STURM_SEQUENCE_HPP
#define STURM_SEQUENCE_HPP

#include <analysis/Polynomial1D.hpp>

#include <vector>

class SturmSequence
{
 private:
  std::vector<Polynomial1D> m_sequence;

 public:
  PUGS_INLINE
  const Polynomial1D&
  p(const size_t i) const
  {
    Assert(i < m_sequence.size());
    return m_sequence[i];
  }

  PUGS_INLINE
  size_t
  size() const
  {
    return m_sequence.size();
  }

  size_t numberOfSignChanges(const double x) const;

  SturmSequence(const Polynomial1D& p0);
  SturmSequence(Polynomial1D&& p0);

  SturmSequence(SturmSequence&&)      = delete;
  SturmSequence(const SturmSequence&) = delete;

  ~SturmSequence() = default;
};

#endif   // STURM_SEQUENCE_HPP
