#include <catch2/catch_approx.hpp>
#include <catch2/catch_test_macros.hpp>

#include <analysis/Polynomial1DRootComputer.hpp>

// clazy:excludeall=non-pod-global-static

TEST_CASE("Polynomial1DRootComputer", "[analysis]")
{
  SECTION("No root in interval")
  {
    Polynomial1D p({-1, 0, 0, 1});

    Polynomial1DRootComputer computer(p, -4, -3);

    std::optional optional_root = computer.getFirstRoot();

    REQUIRE(optional_root.has_value() == false);
  }

  SECTION("one root in interval")
  {
    Polynomial1D p({-1, 0, 0, 1});

    Polynomial1DRootComputer computer(p, -0.3, 1.2);

    std::optional optional_root = computer.getFirstRoot();

    REQUIRE(optional_root.has_value() == true);
    REQUIRE(optional_root.value() == Catch::Approx(1));
  }

  SECTION("multiple roots in interval")
  {
    Polynomial1D p =
      Polynomial1D{{-0.5, 1}} * Polynomial1D{{-0.23, 1}} * Polynomial1D{{-0.27, 1}} * Polynomial1D{{-0.43, 1}};

    Polynomial1DRootComputer computer(p, -0.3, 1.2);

    std::optional optional_root = computer.getFirstRoot();

    REQUIRE(optional_root.has_value() == true);
    REQUIRE(optional_root.value() == Catch::Approx(0.23));
  }
}
